"""Sync version in pyproject.toml with __version__."""
import json
import re
from pathlib import Path

import logzero
import tomlkit
from logzero import logger

from sync_version.loglevel import loglevel

# set LOGLEVEL=10 (Windows) or export LOGLEVEL=10 (Linux) in ternimal to debug
# logzero.loglevel(loglevel(20))

logzero.loglevel(loglevel())
logger.debug("debug on")


def convert(text: str) -> str:
    """Convert version in pyproject.toml to version used in __version__.py.

    s.split("-")[0] + "".join([] if "-" not in s else [s.split("-")[-1][0] + s.split("-")[-1][-1]])

    >>> convert("0.1.0")
    '0.1.0'
    >>> convert("0.1.0-alpha0")
    '0.1.0a0'
    """
    _ = text.split("-")
    return _[0] + "".join([] if "-" not in text else [_[-1][0] + _[-1][-1]])


def fetch_info():
    """Fetch name and version from pyproject.toml."""
    try:
        with open("pyproject.toml", encoding="utf8") as fha:
            jdata = tomlkit.load(fha)
    except Exception as exc:
        logger.error(exc)
        logger.info(
            "You need to run sync-version in the directory where pproject.toml and/or package.json is placed"
        )
        raise SystemExit(1)

    try:
        name = jdata.get("tool").get("poetry").get("name")  # type: ignore
    except Exception:
        name = ""

    try:
        version = jdata.get("tool").get("poetry").get("version")  # type: ignore
    except Exception:
        # check package.json
        logger.warning(
            "No info in pyproject.toml on version available, ... checking package.json "
        )
        try:
            jdata = json.load(open("package.json", encoding="utf8"))
            version = jdata.get("version")
        except Exception:
            logger.warning(
                "No info in package.json on version available, either... setting tp ''(empty) "
            )
            version = ""

    return name, convert(version)


def sync_version(debug: bool = False, dry_run: bool = False):
    """Sync version in pyproject.toml with __version__.

    Args"
        debug: if True set loglevel to 10.

    Typical rundowns:

    poetry version prerelease
    sync-version

    poetry version patch
    sync-version
    """
    try:
        name, version = fetch_info()
    except Exception as exc:
        logger.exception(exc)
        raise

    init_py = Path(name.replace("-", "_")) / "__init__.py"
    if not init_py.is_file():
        logger.warning("File [%s] does not exist or is not a file.", init_py)
        raise SystemExit(1)

    try:
        text = init_py.read_text(encoding="utf8")
    except Exception as exc:
        logger.exception(exc)
        raise

    logger.debug("version: %s", version)

    try:
        text1 = re.sub(
            r"""(__version__\s*=\s*["'])([\w_.-]+)""", rf"\g<1>{version}", text, count=1
        )
    except Exception as exc:
        logger.exception(exc)
        raise

    assert version in text1, "Something is not right...check it out."

    # write back to __init__.py
    if text1.strip() != text.strip():
        logger.info(" %s written to __init__.py.", version)
        if not dry_run:
            try:
                init_py.write_text(text1, encoding="utf8")
            except Exception as exc:
                logger.exception(exc)
                raise
        else:
            logger.info("Dry-run: nothing changed")
    else:
        logger.info("__init__.py remains unchanged.")

    if debug:
        logger.debug("__init__.py: \n%s", text1)
    else:
        logger.info(
            "__init__.py.splitlines()[:4]: \n%s", "\n".join(text1.splitlines()[:4])
        )


if __name__ == "__main__":
    sync_version()



"""Prep __main__ entry."""
from typing import Optional

import logzero
import typer
from logzero import logger

from sync_version import __version__, sync_version
from sync_version.loglevel import loglevel

logzero.loglevel(loglevel())

app = typer.Typer(
    name="sync-version",
    add_completion=False,
    help="pip install robot",
)


def _version_callback(value: bool) -> None:
    if value:
        typer.echo(f"{app.info.name} v.{__version__}")
        raise typer.Exit()


@app.command()
def main(
    dry_run: bool = typer.Option(
        False,
        "--dry-run",
        "-y",
        is_flag=True,
        help="Dry-run: wont write to __init__py.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        is_flag=True,
        help="Turn on debug mode. Note that debug can also be turned on by 'set LOGLEVEL=debug' or 'export LOGLEVEL=debug'.",
    ),
    version: Optional[bool] = typer.Option(  # pylint: disable=(unused-argument
        None,
        "--version",
        "-v",
        "-V",
        help="Show version info and exit.",
        callback=_version_callback,
        is_eager=True,
    ),
):
    """Sync __version__ in __init__py with version in pyproject.toml.

    e.g.

    poetry version prerelease
    # in the directory where pyproject.toml is located
    sync-version  # or ython -m sync_version

    poetry version patch
    sync-version  # or ython -m sync_version

    """
    if debug:
        logger.info("debug is on")
        # logzero.loglevel(loglevel(10, force=True))
        # logzero.loglevel(loglevel(20, force=True))
        logzero.loglevel(loglevel(10))

    logger.debug("debug")

    try:
        sync_version(dry_run=dry_run, debug=debug)
    except Exception as exc:
        logger.error(exc)
        typer.Exit(1)


if __name__ == "__main__":
    # set LOGLEVEL=10 or debug or DEBUG to turn on debug
    logger.debug(" debug on (main)")
    app()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sync_version']

package_data = \
{'': ['*']}

install_requires = \
['environs>=9.5.0,<10.0.0',
 'icecream>=2.1.1,<3.0.0',
 'install>=1.3.5,<2.0.0',
 'logzero>=1.7.0,<2.0.0',
 'tomlkit>=0.10.1,<0.11.0',
 'typer>=0.4.1,<0.5.0']

entry_points = \
{'console_scripts': ['sync-version = sync_version.__main__:app']}

setup_kwargs = {
    'name': 'sync-version',
    'version': '0.1.5a0',
    'description': 'Sync __version__ in __init__.py with version in pyproject.toml',
    'long_description': '# sync-version\n[![pytest](https://github.com/ffreemt/sync-version/actions/workflows/routine-tests.yml/badge.svg)](https://github.com/ffreemt/sync-version/actions)[![python](https://img.shields.io/static/v1?label=python+&message=3.8%2B&color=blue)](https://www.python.org/downloads/)[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)[![PyPI version](https://badge.fury.io/py/sync_version.svg)](https://badge.fury.io/py/sync_version)\n\nsync version in pyproject.toml (if pyproject.toml does not have version info, then version in package.json) with __version__ in __init__.py \n\n## Install it\n\n```shell\npip install sync-version\n\n# or pip install git+https://github.com/ffreemt/sync-version\n# poetry add git+https://github.com/ffreemt/sync-version\n# git clone https://github.com/ffreemt/sync-version && cd sync-version\n```\n\n## Use it\n```python\n\npoetry version prerelease\nsync-version  # or python -m sync_version\n\npoetry version patch\nsync-version  # or python -m sync_version\n\n# dry-run\nsync-version --dry-run\n\n# debug and dry-run\nsync-version --debug --dry-run\n\n```\n',
    'author': 'ffreemt',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/ffreemt/sync-version',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.3,<4.0.0',
}


setup(**setup_kwargs)
